import flet as ft
import time
import base64
import sqlite3
from gui.widgets import NavBar, ConfirmDialog, ErrorDialog, SearchBar
from book.Book import BookController

class BookContainer(ft.Container):
    def __init__(self, booklist: ft.GridView, isbn:str, title:str, cover:bytes | None, status: str, quantity: int):
        super().__init__()
        self.booklist = booklist
        self.title = title
        self.isbn = isbn
        self.cover = cover
        self.disp_cover = ft.Image(
            width=100,
            height=160,
            fit=ft.ImageFit.CONTAIN
        )
        if cover == None:
            self.disp_cover.src = "src/assets/covers/nocover.png"
        else:
            self.disp_cover.src_base64 = (base64.b64encode(cover)).decode("utf-8")

        self.status = status
        self.quantity = quantity

        self.editButton = ft.IconButton(
            icon=ft.icons.EDIT,
            bgcolor=ft.colors.DEEP_PURPLE_ACCENT,
            icon_color=ft.colors.WHITE,
            tooltip="Edit book",
            on_click=self.edit_book
        )
        self.deleteButton = ft.IconButton(
            icon=ft.icons.DELETE,
            bgcolor=ft.colors.RED_ACCENT,
            icon_color=ft.colors.WHITE,
            tooltip="Delete book",
            on_click=self.delete_book
        )

        self.bgcolor = ft.colors.INDIGO_900
        self.padding = 10
        self.content = ft.Row(
            width=500,
            height=100,
            spacing=20,
            controls=[
                self.disp_cover,
                ft.Column(
                    expand=3,
                    controls=[
                        ft.Text(title, size=20, weight=ft.FontWeight.BOLD),
                        ft.Text("ISBN: " + isbn),
                        ft.Text("Status: " + status),
                        ft.Text("Quantity: " + str(quantity))
                    ]
                ),
                ft.Column(
                    controls=[
                        self.editButton, self.deleteButton
                    ]
                )
            ]
        )
    
    def edit_book(self, e: ft.TapEvent):
        dlg = BookEdit(self, self.booklist.page)
        dlg.open_dlg(e)
        while dlg.dlg.open:
            time.sleep(.01)
        if dlg.doAction:
            if int(dlg.newQuantity.value) < 0:
                err = ErrorDialog(self.page, "Insert failed. Book quantity cannot be negative.")
                err.open_dlg(e)
            elif dlg.newTitle.value == None:
                dlg.newTitle.value = self.title
            elif dlg.newQuantity.value == None:
                dlg.newQuantity.value = self.quantity
            else:
                book_controller = BookController(sqlite3.connect('src/db/library.db'))
                book_controller.editBook(self.isbn, dlg.newTitle.value, dlg.newCover, self.status, int(dlg.newQuantity.value))
                self.booklist.update_data()
    
    def delete_book(self, e: ft.TapEvent):
        dlg = ConfirmDialog(self.booklist.page, "delete book")
        dlg.open_dlg(e)
        while dlg.dlg.open:
            time.sleep(.01)
        if dlg.doAction:
            if self.status == "loaned":
                err = ErrorDialog(self.page, "Delete failed. Book cannot be deleted because it is being loaned.")
                err.open_dlg(e)
            else:
                book_controller = BookController(sqlite3.connect('src/db/library.db'))
                book_controller.removeBook(self.isbn)
                self.booklist.update_data()

class BookEdit():
    def __init__(self, book: ft.Container, page: ft.Page):
        self.book = book
        self.page = page
        self.doAction = False
        self.newCover = self.book.cover

        self.newTitle = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250,
            value=self.book.title
        )
        self.newQuantity = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250,
            value=self.book.quantity
        )

        self.newTitleRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Title: "), self.newTitle
            ]
        )
        self.newQuantityRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Quantity: "), self.newQuantity
            ]
        )

        self.filePicker = ft.FilePicker(on_result=self.set_image)
        self.page.overlay.append(self.filePicker)
        self.filePickerButton = ft.TextButton(
            text="Choose cover",icon=ft.icons.UPLOAD,
            on_click=lambda _:self.filePicker.pick_files(file_type=ft.FilePickerFileType.IMAGE)
        )

        self.dlg = ft.AlertDialog(
            modal=False,
            title=ft.Text("Edit Book"),
            content=ft.Column(
                width=400,
                height=200,
                controls=[
                    ft.Text("ISBN: "+self.book.isbn),
                    ft.Text("Status: "+self.book.status),
                    self.newTitleRow,
                    self.newQuantityRow,
                    self.filePickerButton
                ]
            ),
            actions=[
                ft.TextButton("Cancel", on_click=self.close_dlg),
                ft.TextButton("Save", on_click=self.close_dlg_do),
            ],
            actions_alignment=ft.MainAxisAlignment.END,
            bgcolor=ft.colors.BLACK
        )
    
    def close_dlg(self, e: ft.TapEvent):
        self.dlg.open = False
        self.page.update()
    
    def close_dlg_do(self, e: ft.TapEvent):
        self.doAction = True
        self.dlg.open = False
        self.page.update()
    
    def open_dlg(self, e: ft.TapEvent):
        self.page.dialog = self.dlg
        self.dlg.open = True
        self.page.update()
    
    def set_image(self, e: ft.FilePickerResultEvent):
        if e.files:
            with open(e.files[0].path, 'rb') as img:
                self.newCover = img.read()

class BookAdd():
    def __init__(self, page: ft.Page):
        self.page = page
        self.doAction = False
        self.newCover = None
        self.newISBN = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250,
        )
        self.newTitle = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250,
        )
        self.newQuantity = ft.TextField(
            bgcolor=ft.colors.GREY_900,
            border_color=ft.colors.WHITE,
            height=30,
            text_size=15,
            content_padding=10,
            cursor_color=ft.colors.WHITE,
            width=250,
        )
        self.newISBNRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("ISBN: "), self.newISBN
            ]
        )
        self.newTitleRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Title: "), self.newTitle
            ]
        )
        self.newQuantityRow = ft.Row(
            alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
            controls=[
                ft.Text("Quantity: "), self.newQuantity
            ]
        )

        self.filePicker = ft.FilePicker(on_result=self.set_image)
        self.page.overlay.append(self.filePicker)
        self.filePickerButton = ft.TextButton(
            text="Choose cover",icon=ft.icons.UPLOAD,
            on_click=lambda _:self.filePicker.pick_files(file_type=ft.FilePickerFileType.IMAGE)
        )

        self.dlg = ft.AlertDialog(
            modal=False,
            title=ft.Text("New Book"),
            content=ft.Column(
                width=400,
                height=200,
                controls=[
                    self.newISBNRow,
                    self.newTitleRow,
                    self.newQuantityRow,
                    self.filePickerButton
                ]
            ),
            actions=[
                ft.TextButton("Cancel", on_click=self.close_dlg),
                ft.TextButton("Add", on_click=self.close_dlg_do),
            ],
            actions_alignment=ft.MainAxisAlignment.END,
            bgcolor=ft.colors.BLACK
        )
    
    def close_dlg(self, e: ft.TapEvent):
        self.dlg.open = False
        self.page.update()
    
    def close_dlg_do(self, e: ft.TapEvent):
        self.doAction = True
        self.dlg.open = False
        self.page.update()
    
    def open_dlg(self, e: ft.TapEvent):
        self.page.dialog = self.dlg
        self.dlg.open = True
        self.page.update()
    
    def set_image(self, e: ft.FilePickerResultEvent):
        if e.files:
            with open(e.files[0].path, 'rb') as img:
                self.newCover = img.read()

class BookList(ft.GridView):
    def __init__(self):
        super().__init__()
        self.child_aspect_ratio = 2.5
        self.runs_count = 2
    def update_data(self):
        book_controller = BookController(sqlite3.connect('src/db/library.db'))
        self.spacing = 10
        self.controls = []
        for b in book_controller.displayBook():
            bc = BookContainer(self, b[0], b[1], b[2], b[3], b[4])
            self.controls.append(bc)
        self.update()

    def filter_data(self, input):
        book_controller = BookController(sqlite3.connect('src/db/library.db'))
        self.spacing = 10
        self.controls = []
        for b in book_controller.searchBook(input):
            bc = BookContainer(self, b[0], b[1], b[2], b[3], b[4])
            self.controls.append(bc)
        self.update()

class BookView(ft.View):
    def __init__(self, page:ft.Page):
        super().__init__()
        self.page = page
        self.route = "/books"
        self.scroll = True
        self.books = BookList()
        self.search = SearchBar(self.books)
        self.add = ft.ElevatedButton(text="New", icon=ft.icons.ADD, bgcolor=ft.colors.GREEN, color=ft.colors.WHITE, on_click=self.add_book)
        self.controls=[
            ft.Column(
                horizontal_alignment=ft.CrossAxisAlignment.CENTER,
                expand=True,
                controls=[
                    NavBar(page),
                    ft.Text("BOOK LIST", weight=ft.FontWeight.BOLD, size=22),
                    ft.Row(
                        alignment=ft.MainAxisAlignment.SPACE_BETWEEN,
                        controls=[
                            self.search, self.add
                        ]
                    ),
                    self.books
                ]
            )
        ]
    def add_book(self, e: ft.TapEvent):
        dlg = BookAdd(self.page)
        dlg.open_dlg(e)
        while dlg.dlg.open:
            time.sleep(.01)
        if dlg.doAction:
            book_controller = BookController(sqlite3.connect('src/db/library.db'))
            if dlg.newTitle.value == None or dlg.newISBN.value == None or dlg.newQuantity.value == None:
                err = ErrorDialog(self.page, "Insert failed. All fields must be filled.")
                err.open_dlg(e)
            elif book_controller.getBook(dlg.newISBN.value) != None:
                err = ErrorDialog(self.page, "Insert failed. Book already exists.")
                err.open_dlg(e)
            else:
                book_controller.addBookByFields(dlg.newISBN.value, dlg.newTitle.value, dlg.newCover, dlg.newQuantity.value)
                self.books.update_data()