from datetime import date
import sqlite3

class Loan:
    def __init__(self, ID: int, bookISBN: str, memberID: int, borrowDate: date, returnDate: date):
        self.__ID = ID
        self.__bookISBN = bookISBN
        self.__memberID = memberID
        self.__borrowDate = borrowDate
        self.__returnDate = returnDate

    def getID(self) -> int:
        return self.__ID

    def getBookISBN(self) -> str:
        return self.__bookISBN

    def getMemberID(self) -> int:
        return self.__memberID

    def getBorrowDate(self) -> date:
        return self.__borrowDate

    def getReturnDate(self) -> date:
        return self.__returnDate

class LoanController:
    def __init__(self, connection):
        self.connection = connection
        self.create_table()

    def create_table(self):
        cursor = self.connection.cursor()
        cursor.execute('''
            CREATE TABLE IF NOT EXISTS Loan (
                ID INTEGER PRIMARY KEY AUTOINCREMENT,
                BookISBN TEXT NOT NULL,
                MemberID INTEGER NOT NULL,
                BorrowDate DATE NOT NULL,
                ReturnDate DATE NOT NULL,
                CONSTRAINT fk_book
                    FOREIGN KEY (BookISBN)
                    REFERENCES Book(ISBN)
                    ON DELETE RESTRICT,
                CONSTRAINT fk_member
                    FOREIGN KEY (MemberID)
                    REFERENCES Member(ID)
                    ON DELETE CASCADE
            )
        ''')
        self.connection.commit()

    def addLoan(self, peminjaman):
        cursor = self.connection.cursor()
        cursor.execute('''
            SELECT * FROM Loan Where MemberID=?''', (peminjaman.getMemberID(),))
        loan = cursor.fetchall()
        if (len(loan) >=3):
            print("Member tidak bisa meminjam lebih dari tiga buku")
            return
        cursor.execute('''
            INSERT INTO Loan (ID, BookISBN, MemberID, BorrowDate, ReturnDate)
            VALUES (?, ?, ?, ?, ?)
        ''', (peminjaman.getID(), peminjaman.getBookISBN(), peminjaman.getMemberID(), peminjaman.getBorrowDate(), peminjaman.getReturnDate()))

        cursor.execute('''
            UPDATE Book SET Status = ?, Quantity = Quantity - 1 WHERE ISBN = ?
        ''', ("loaned", peminjaman.getBookISBN()))
        self.connection.commit()

    def addLoanByFields(self, book, member, borrowDate, returnDate):
        cursor = self.connection.cursor()
        # cursor.execute('''
        #     SELECT * FROM Loan Where MemberID=?''', (member,))
        # loan = cursor.fetchall()
        # if (len(loan) >=3):
        #     print("Member tidak bisa meminjam lebih dari tiga buku")
        #     return
        cursor.execute('''
            INSERT INTO Loan (BookISBN, MemberID, BorrowDate, ReturnDate)
            VALUES (?, ?, ?, ?)
        ''', (book, member, borrowDate, returnDate,))
        cursor.execute('''
            UPDATE Book SET Status = ?, Quantity = Quantity - 1 WHERE ISBN = ?
        ''', ("loaned", book,))
        cursor.execute('''
            UPDATE Member SET booksBorrowed = booksBorrowed + 1 WHERE ID = ?
        ''', (member,))
        self.connection.commit()

    def checkLoan(self, loan_id):
        cursor = self.connection.cursor()
        cursor.execute('''
            SELECT * FROM Loan WHERE ID = ?
        ''', (loan_id,))
        return cursor.fetchone()

    def displayLoan(self):
        cursor = self.connection.cursor()
        cursor.execute('''
            SELECT * FROM Loan
        ''')
        return cursor.fetchall()

    def editLoan(self, loan_id, new_borrow_date, new_return_date):
        cursor = self.connection.cursor()
        cursor.execute('''
            UPDATE Loan SET BorrowDate = ?, ReturnDate = ? WHERE ID = ?
        ''', (new_borrow_date, new_return_date, loan_id))
        self.connection.commit()

    def returnBook(self, loan_id):
        cursor = self.connection.cursor()

        # Get book ISBN from loan
        cursor.execute('''
            SELECT BookISBN FROM Loan WHERE ID = ?
        ''', (loan_id,))
        bookISBN = cursor.fetchone()[0]

        # Remove loan
        cursor.execute('''
            DELETE FROM Loan WHERE ID = ?
        ''', (loan_id,))

        # Increment the book quantity in Book table
        cursor.execute('''
            UPDATE Book SET Quantity = Quantity + 1 WHERE ISBN = ?
        ''', (bookISBN,))

        if self.getLoanByBook(bookISBN) == None:
            cursor.execute('''
                UPDATE Book SET Status = ? WHERE ISBN = ?
            ''', ("not loaned", bookISBN,))

        cursor.execute('''
            SELECT MemberID FROM Loan WHERE ID = ?
        ''', (loan_id,))
        memberID = cursor.fetchone()[0]

        cursor.execute('''
            UPDATE Member SET booksBorrowed = booksBorrowed - 1 WHERE ID = ?
        ''', (memberID,))

        self.connection.commit()

    def getLoan(self, loan_id):
        cursor = self.connection.cursor()
        cursor.execute('''
            SELECT * FROM Loan WHERE ID = ?
        ''', (loan_id,))
        return cursor.fetchone()
    
    def getLoanByBook(self, bookISBN):
        cursor = self.connection.cursor()
        cursor.execute('''
            SELECT * FROM Loan WHERE BookISBN = ?
        ''', (bookISBN,))
        return cursor.fetchone()
    
    def searchLoan(self, input):
        wildc_input = "%" + input + "%"
        cursor = self.connection.cursor()
        cursor.execute('''
            SELECT * FROM Loan WHERE BookISBN LIKE ? OR CAST(MemberID AS VARCHAR) LIKE ?
        ''', (wildc_input, wildc_input,))
        return cursor.fetchall()


# Example Usage
if __name__ == "__main__":

    connection = sqlite3.connect('../db/library.db')

    peminjaman = Loan(1, '978-3-16-148410-0', 12345, date(2024, 5, 1), date(2024, 5, 14))

    loan_controller = LoanController(connection)

    # loan_controller.addLoan(peminjaman)

    loan = loan_controller.getLoan(1)
    print("Loan Details:", loan)

    loans = loan_controller.displayLoan()
    for loan in loans:
        print(loan)

    loan_controller.editLoan(1, date(2024, 5, 2), date(2024, 5, 15))

    # loan_controller.returnBook(1)

    connection.close()
